########################################################################
#updateTM_TCResults1.pl
#
# Updates the interpreted result for logs that have no test cases 
# assigned to them, e.g. defect scripts.
#
########################################################################

use PjCLib;
use Win32::OLE;
require CQPerlExt;


################
# main
################

# Global variables
$currentDate; # SQL date format
$currentTime; # SQL time format

# Initialize the date/time variables.
getCurrentDateTime();

# Get the ProjectConsole CQ session variable.
$session = logon();

myMain();
exit;

sub myMain {

    # You must always call the initializePjCLibrary() first before
    # you can use the PjCLib.
    PjCLib->initializePjCLibrary($session, PjCLib->BOOLEAN_TRUE);

    ######################################################################
    #  Your transformation code goes here
	
	setTableAndFieldNames();
	updateLogsWithNoTests();
    #
    ######################################################################
} # end subroutine myMain()

sub setTableAndFieldNames () {

    # Set the names of the physical tables.

    $RESULTS_DIM_TABLE = PjCLib->getPhysicalTableDbName("TM_InterpretedResult_Dim");
    $LOG_EVENT_RESULT_TABLE = PjCLib->getPhysicalTableDbName("TM_LogFirstRootEventResult_Dim");
    $TC_RESULTS_MEASURE_TABLE = PjCLib->getPhysicalTableDbName("TM_TCResults_Measure");
    print "RESULTS_DIM_TABLE: $RESULTS_DIM_TABLE\n";
    print "LOG_EVENT_RESULT_TABLE: $LOG_EVENT_RESULT_TABLE\n";
    print "TC_RESULTS_MEASURE_TABLE: $TC_RESULTS_MEASURE_TABLE\n";

    # Set the names of the physical fields for the $RESULTS_DIM_TABLE.

    $TM_RESULT_NAME = PjCLib->getPhysicalFieldDbName("TM_InterpretedResult_Dim", "TM_InterpretedResult");
    print "TM_RESULT_NAME: $TM_RESULT_NAME\n";

    # Set the names of the physical fields for the $LOG_EVENT_RESULT_TABLE.

    $TM_LOG_EVENT_RESULT = PjCLib->getPhysicalFieldDbName("TM_LogFirstRootEventResult_Dim", "TM_LogFirstRootEventResult");
    print "TM_LOG_EVENT_RESULT: $TM_LOG_EVENT_RESULT\n";

    # Set the names of the physical fields for the $TC_RESULTS_MEASURE_TABLE.

    $BUILD_NAME = PjCLib->getPhysicalFieldDbName("TM_TCResults_Measure", "TM_BuildName_fk");
    $BUILD_UID = PjCLib->getPhysicalFieldDbName("TM_TCResults_Measure", "TM_BuildUID_fk");
    $LATEST_COLLECTION = PjCLib->getPhysicalFieldDbName("TM_TCResults_Measure", "IsLatestCollection");
    $LOG_NAME = PjCLib->getPhysicalFieldDbName("TM_TCResults_Measure", "TM_LogName_fk");
    $LOG_UID = PjCLib->getPhysicalFieldDbName("TM_TCResults_Measure", "TM_LogUID_fk");
    $INTERPRETED_RESULT = PjCLib->getPhysicalFieldDbName("TM_TCResults_Measure", "TM_InterpretedResult_fk");
    $LOG_EVENT_RESULT = PjCLib->getPhysicalFieldDbName("TM_TCResults_Measure", "TM_LogFirstRootEventResult_fk");
    print "BUILD_NAME: $BUILD_NAME \n";
    print "BUILD_UID = $BUILD_UID\n";
    print "LATEST_COLLECTION: $LATEST_COLLECTION\n";
    print "LOG_NAME: $LOG_NAME\n";
    print "LOG_UID: $LOG_UID\n";
    print "INTERPRETED_RESULT: $INTERPRETED_RESULT\n";
    print "LOG_EVENT_RESULT: $LOG_EVENT_RESULT\n";

} # end subroutine setTableAndFieldNames


sub updateLogsWithNoTests()
{
    # Declare local variables used by subroutine.

    my($queryStr);           # String used to query the CQ database
    my($resultSet); 	     # Results from executing a SQL query of $tableName
    my($resultSetWithTests); # Results from 
    my($resultSetForTCStatus);
    my($dbid);               # dbid of an existing record in the $TOTALS_BY_LOG_AND_BUILD_TABLE
    my($entity);             # Record to delete from $TOTALS_BY_LOG_AND_BUILD_TABLE
    my($retval);             # Return value after deleting record from $TOTALS_BY_LOG_AND_BUILD_TABLE
    my($buildName);          # Name of the build 
    my($logName);            # Name of the log
    my($interpretedResult);   # Interpreted result for build & log
    my($logFirstRootEventResult);

    # Determine if the measure table contains any logs that have a result but have no test cases in them.

    $queryStr = "SELECT $TC_RESULTS_MEASURE_TABLE.dbid, $TC_RESULTS_MEASURE_TABLE.$BUILD_UID, ".
                "$TC_RESULTS_MEASURE_TABLE.$LOG_UID, $TC_RESULTS_MEASURE_TABLE.$INTERPRETED_RESULT, " .
                "$LOG_EVENT_RESULT_TABLE.$TM_LOG_EVENT_RESULT " .
		"FROM $TC_RESULTS_MEASURE_TABLE INNER JOIN " .
                "$LOG_EVENT_RESULT_TABLE ON $TC_RESULTS_MEASURE_TABLE.$LOG_EVENT_RESULT = $LOG_EVENT_RESULT_TABLE.dbid ".
		"WHERE ($TC_RESULTS_MEASURE_TABLE.$INTERPRETED_RESULT = 0) AND " .
                "($TC_RESULTS_MEASURE_TABLE.$LOG_EVENT_RESULT <> 0) AND " .
	        "($TC_RESULTS_MEASURE_TABLE.$LATEST_COLLECTION = 1) ";
 
    print "queryStr = $queryStr\n";
       
    $resultSet = $session->BuildSQLQuery($queryStr);
    $resultSet->Execute();

    while ($resultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
	$dbid = $resultSet->GetColumnValue(1);
        $buildUID = $resultSet->GetColumnValue(2);
        $logUID = $resultSet->GetColumnValue(3);
        $logFirstRootEventResult = $resultSet->GetColumnValue(5);
        print "dbid: $dbid\n";
    	print "buildUID: $buildUID\n";
        print "logUID: $logUID\n";
        print "logFirstRootEventResult: $logFirstRootEventResult\n";      
      
    	# Determine if the current build & log have tests associated with it.

	$queryStr = "SELECT COUNT($TC_RESULTS_MEASURE_TABLE.$INTERPRETED_RESULT) AS NumOfTests, " .
                    "$TC_RESULTS_MEASURE_TABLE.$BUILD_UID, $TC_RESULTS_MEASURE_TABLE.$LOG_UID " .
		    "FROM $TC_RESULTS_MEASURE_TABLE " .
		    "WHERE ($TC_RESULTS_MEASURE_TABLE.$LOG_EVENT_RESULT <> 0) AND " .
		    "($TC_RESULTS_MEASURE_TABLE.$LATEST_COLLECTION = 1) ".
		    "GROUP BY $BUILD_UID, $LOG_UID, $INTERPRETED_RESULT " .
		    "HAVING ($BUILD_UID = $buildUID) AND ($LOG_UID = $logUID) AND ($INTERPRETED_RESULT <> 0)";

        print "queryStr = $queryStr\n";

    	$resultSetWithTests = $session->BuildSQLQuery($queryStr);
	$resultSetWithTests->Execute();

	if ($resultSetWithTests->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
	    # There are tests associated with this build & log. There is no need to update the record.
            print "no update\n";
	}
	else {

	    # Determine if the result exists in the interpreted results table.

   	    $queryStr = "SELECT * FROM $RESULTS_DIM_TABLE WHERE $TM_RESULT_NAME = '$logFirstRootEventResult'";
  	    
            $resultSetForTCStatus = $session->BuildSQLQuery($queryStr);
	    $resultSetForTCStatus->Execute();

   	    if ($resultSetForTCStatus->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
	        # The test case status already exists in the interpreted results table.
        	print "result already exists\n";
	    }
	    else {
	        # Insert a new record into the interpreted results table.
 	    
   	        $entity = $session->BuildEntity($RESULTS_DIM_TABLE);
                $entity->SetFieldValue($TM_RESULT_NAME, $logFirstRootEventResult);
                $retVal = $entity->Validate();
                print "interpreted results entity->Validate() return value = $retVal\n";
                $retVal = $entity->Commit();
                print "interpreted results entity->Commit() return value = $retVal\n";		    				
	    }
	    
            $entity = $session->GetEntity("$TC_RESULTS_MEASURE_TABLE", "$dbid"); 
            $session->EditEntity($entity, "modify");
	    $entity->SetFieldValue("$INTERPRETED_RESULT", $logFirstRootEventResult);
            $retVal = $entity->Validate();
            print "entity->Validate() return value = $retVal\n";
            $retVal = $entity->Commit();
            print "entity->Commit() return value = $retVal\n";
        }
    }
}


sub logon {

    #
    # Use the internal DashboardSystem login to get access to the Dashboard database
    #
    my $session = Win32::OLE->new('CLEARQUEST.SESSION');
    my $session = CQPerlExt::CQSession_Build();  
    $session->UserLogon("dashboardsystem", "dashboardsystem", "tstd1", "Dashboard");
    return $session;
}

# get date and set the global variables $currentDate and $currentTime
sub getCurrentDateTime {
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);

    $mon = $mon + 1;
    $year = $year + 1900;

    if ($mday<10) {$mday="0$mday"; }  # add leading 0
    if ($mon<10) {$mon="0$mon"; }     # add leading 0

    # set the global variable $currentDate
    $currentDate="$year-$mon-$mday";

    # set the global variable $currentTime
    $currentTime = $hour . ":" . $min . ":" . $sec;

}

